#include <stdio.h>
#include <stdlib.h>

inline int readInt(unsigned int*);

unsigned int partition(unsigned int* array, unsigned int left, unsigned int right);
void quicksort(unsigned int* array, unsigned int left, unsigned int right);

inline unsigned int isSumOdd (unsigned int*);
unsigned int* multisetUnion (unsigned int* first, unsigned int* second, unsigned int m);
void sumAllCountUnique(unsigned int* set, unsigned int* sum, unsigned int* count);

int main(void){
	unsigned int n, m, k;
	unsigned int i, j, length, second, third;
	unsigned int** sets; /* [n] Remembers pointers to all multisets */
	unsigned int *first; 
	unsigned int sumAll, countUnique;
	
	/* Read in:
	n - number of multisets (sets with repeating elements),
	m - max number of elements of a single multisets' union,
	k - number of iterations of multisets' union */
	readInt(&n);
	readInt(&m);
	readInt(&k);
	
	/* Allocate memory for 'sets' array */
	sets = (unsigned int**)malloc(n * sizeof(unsigned int*));
	
	/* Read in n multisets' elements, allocate memory.
	Remember no. elements in i-th multiset in set[i][0] */
	for(i = 0; i < n; i++){
		readInt(&length);
		sets[i] = (unsigned int*)malloc((length+1) * sizeof(unsigned int));
		sets[i][0] = length;
		for(j = 1; j <= length; j++){
			readInt(sets[i] + j);
		}
		/* Sort i-th set */
		quicksort(sets[i], 1, length);
	}
	
	/* Initially set up 'first' to be an empty multiset */
	first = (int*)malloc(sizeof(int));
	first[0] = 0;
	
	/* Perform k iterations of multisets' union */
	for(i = 0; i < k; i++){
		readInt(&second);
		readInt(&third);
		if(isSumOdd(first)){
		/* If the sum of 'first' elements is odd, union with 'third' set */
			first = multisetUnion(first, sets[third], m);
		}
		else{
		/* If the sum of 'first' elements is even, union with 'second' set */
			first = multisetUnion(first, sets[second], m);
		}
	}
	/* Sum all elements of 'first' and count no. unique elements */
	sumAllCountUnique(first, &sumAll, &countUnique);
	
	printf("%d %d\n", sumAll, countUnique);
	
	/* Free allocated memory */
	free(first);
	for(i = 0; i < n; i++)
		free(sets[i]);
	free(sets);
	
	return 0;
}

inline int readInt(unsigned int* num)
{
    int digit;
    /* Read the first character */
    digit = getchar();	
    *num = digit - '0';
    /* Read in characters until encountering either EOF or a whitespace character (space or new line)*/
    while((digit = getchar()) != EOF){
        if(digit == 32 || digit == 10)
            return digit;
        /* Multiply number by 10 using bitwise operations: 10x = 8x + 2x */
        *num = ((*num)<<3) + ((*num)<<1);
        /* Add new digit as the least significant one */
        *num = *num + digit - '0';
    }
    return 0;
}

/* QUICKSORT */
unsigned int partition(unsigned int* array, unsigned int left, unsigned int right)
{
    unsigned int temp;
    unsigned int pivot;
    /* Choose item to use for comparison */
    pivot = *(array+left);
    while(1)
    {
        while(*(array+right) > pivot)
            right--;
        while(*(array+left) < pivot)
            left++;
        if(left < right)
        {
            /* Swap items at indices left and right */
            temp = *(array+left);
            *(array+left) = *(array+right);
            *(array+right) = temp;
            left++;
            right--;
        }
        else
            return right;
    }
}

void quicksort(unsigned int* array, unsigned int left, unsigned int right)
{
    unsigned int pivot;
    if(left < right)
    {
        /* Choose the index to partition array */
        pivot = partition(array, left, right);
        /* Recursive calls for both parts of original array */
        quicksort(array, left, pivot);
        quicksort(array, pivot+1, right);
    }
}

inline unsigned int isSumOdd (unsigned int* set)
{
	unsigned int sum, i, size;
	sum = 0;
	size = *(set);
	for(i = 1; i <= size; i++)
		sum ^= (*(set+i))&1;
	return sum;
}

unsigned int* multisetUnion (unsigned int* first, unsigned int* second, unsigned int m)
{
	unsigned int size, sizeFirst, sizeSecond, i, j, k;
	unsigned int* setUnion;
	
	sizeFirst = *first;
	sizeSecond = *second;
	
	if(sizeFirst + sizeSecond < m)
		size = sizeFirst + sizeSecond;
	else
		size = m;
		
	setUnion = (unsigned int*)malloc((size+1)*sizeof(unsigned int));
	setUnion[0] = size;
	
	k = 1; i = j = 1;
	while (k <= size){
		if(i <= sizeFirst && j <= sizeSecond){
			if(*(first+i) <= *(second+j)){
				setUnion[k++] = *(first+i);
				i++;
			}
			else{
				setUnion[k++] = *(second+j);
				j++;
			}
		}
		else if(i <= sizeFirst){
			setUnion[k++] = *(first+i);
			i++;
		}
		else{
			setUnion[k++] = *(second+j);
			j++;
		}
	}
	free(first);
	return setUnion;
}
void sumAllCountUnique(unsigned int* set, unsigned int* sum, unsigned int* count){
	unsigned int i, size, lastCounted, current;
	*sum = 0;
	*count = 1;
	size = *set;
	lastCounted = *(set+1);
	
	for(i = 1; i <= size; i++){
		current = *(set+i);
		*sum += current;
		if(current != lastCounted){
			(*count)++;
			lastCounted = current;
		}
	}
}

